@@ -21,12 +21,12 @@
 #include "utils.h"
 #include "frame.h"
 #include "sprite.h"
-#include "data.h"
+#include "setting.h"
 #include "board.h"
 #include "bubble.h"
 
-extern Set bubble_animation[NB_COLORS][NB_BUBBLE_STATES];
 extern int scale;
+extern Set bubble_animation[NB_COLORS][NB_BUBBLE_STATES];
 
 Bubble new_bubble( int color, double x, double y, int layer ) {
   Bubble bubble = ( Bubble ) xmalloc( sizeof( struct _Bubble ));
@@ -36,7 +36,8 @@ Bubble new_bubble( int color, double x, double y, int layer ) {
   bubble->color = color;
   bubble->x = x;
   bubble->y = y;
-  bubble->sprite = new_sprite( layer, 30, bubble_animation[color][NEW], 0, 0 );
+  bubble->vx = bubble->vy = 0;
+  bubble->sprite = new_sprite( layer, 30, bubble_animation[color][NEW], 0, 1 );
   set_sprite_position( bubble->sprite, scale_x(x, scale), scale_y(y, scale) );
   return bubble;
 }
@@ -52,7 +53,7 @@ void set_bubble_state( Bubble bubble, enum BubbleState state, int layer,
   s = bubble->sprite;
   bubble->state = state;
   bubble->clock = clock;   
-  set_sprite_animation( s, bubble_animation[bubble->color][state], 0, 0 );
+  set_sprite_animation( s, bubble_animation[bubble->color][state], 0, 1 );
   set_sprite_layer( s, layer);
 }
 
