@@ -3,24 +3,16 @@
 
 #include "sprite.h"
 
-enum BubbleState { 
-  NEW = 0,
-  READY,
-  LAUNCHED, 
-  STUCK, 
-  EXPLODING, 
-  FALLING, 
-  RISING,
-  RUNAWAY,
-  DEAD
-};
-
-#define NB_BUBBLE_STATES (DEAD+1)
-
 struct _Bubble {
   enum BubbleState state;
-  int color, clock, cell;
-  double x, y, vx, vy, target_y;
+  int color;
+  int clock;
+  int cell;
+  double x;
+  double y;
+  double vx;
+  double vy;
+  double target_y;
   Sprite sprite;
 };
 typedef struct _Bubble * Bubble;
