@@ -20,69 +20,110 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <signal.h>
 #include <math.h>
-#include <sys/time.h>
+#include <unistd.h>
+#include <setjmp.h>
 
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
+#include "setting.h"
+#include "gettext.h"
+
 #include "utils.h"
-#include "data.h"
-#include "rectangle.h"
 #include "sprite.h"
 #include "rgba.h"
 #include "board.h"
 #include "opponent.h"
 #include "controls.h"
 #include "dialog.h"
+#include "timer.h"
 #include "game.h"
+#include "cell.h"
 
 extern Display *display;
-extern Window root, win;
-extern int screen, depth;
+extern Window root;
+extern Window win;
+extern int screen;
+extern int depth;
 
-extern GC dialog_gc, dialog_text_gc;
-extern XFontStruct *dialog_font, *menu_font;
-extern Pixmap win_bg, board_bg;
-extern Pixmap cup_on, cup_off, cup_on_mask, cup_off_mask;
+extern GC dialog_gc;
+extern GC dialog_text_gc;
+extern XFontStruct *dialog_font;
+extern XFontStruct *menu_font;
+extern Pixmap win_bg;
+extern Pixmap cup_on;
+extern Pixmap cup_off;
+extern Pixmap cup_on_mask;
+extern Pixmap cup_off_mask;
+extern Pixmap board_decoration;
+extern Pixmap board_decoration_mask;
 
-extern int scale, fps;
+extern int titlefont_color[3];
+extern int menufont_color[3];
+
+extern int scale;
 extern unsigned long frame_duration;
 
-enum GameState { PLAYING, PAUSED, OVER, FINISHED, STOPPED };
-enum GameComputerState { IDLE, THINKING, AIMING }; 
+enum GameState {
+  PLAYING,
+  PAUSED,
+  OVER,
+  FINISHED,
+  STOPPED
+};
+
+enum GameComputerState {
+  IDLE,
+  THINKING,
+  AIMING
+}; 
 
 struct _Game {
   enum GameMode mode;
   enum GameState state;
   enum GameResult result;
   enum GameComputerState computer_state[2];
-  int player_left[2], player_right[2], player_fire[2];
-  int escape_pressed, pause_pressed, key_pressed, multi_player;
-  int nb_boards, handicap[2], board_best_eval[2], board_best_angle[2];
-  int round, computer_thinking_time[2], board_x[2], board_y[2], *score;
-  unsigned int width, height;
+  int player_left[2];
+  int player_right[2];
+  int player_fire[2];
+  int escape_pressed;
+  int pause_pressed;
+  int key_pressed;
+  int multi_player;
+  int nb_boards;
+  int handicap[2];
+  int board_best_eval[2];
+  int board_best_angle[2];
+  int round;
+  int need_restart_thinking;
+  int computer_thinking_time[2];
+  int board_x[2];
+  int board_y[2];
+  int *score;
+  unsigned int width;
+  unsigned int height;
   double canon_angle[2];
   Board board[2];
   Opponent opponent[2];
   GC board_gc;
   Pixmap board_pixmap[2];
-  Window board_window[2], msg_box[2], cup_box[2], tags[2];
+  Pixmap board_bg[2];
+  Window board_window[2];
+  Window msg_box[2];
+  Window cup_box[2];
+  Window tags[2];
+  RuleSet_t ruleset[2];
 };
-Game current_game = NULL;
 
-#define PERIOD        12
-#define PLAYER1_TAG   "Player 1"
-#define PLAYER2_TAG   "Player 2"
-#define COMPUTER_TAG  "Computer"
+Game current_game = NULL;
+sigjmp_buf restart_thinking;
 
 char *player_name[][2] = {
   { PLAYER1_TAG, "" },
   { PLAYER1_TAG, PLAYER2_TAG },
   { PLAYER1_TAG, COMPUTER_TAG },
-  { COMPUTER_TAG, COMPUTER_TAG }};
+  { DEMO_TAG, DEMO_TAG }};
 
 void display_cups( Game game, int n, int on ) {
   
@@ -112,19 +153,26 @@ void display_cups( Game game, int n, int on ) {
   if ( game->score[n] > 0 ) {
     XSetClipMask( display, gc, (( on )? cup_on_mask : cup_off_mask ));
     XSetClipOrigin( display, gc, pos, 0 );
-    XCopyArea( display, (( on )? cup_on : cup_off ), back, gc,
-	       0, 0, scale, height, pos, 0 );
+    XCopyArea( display, (( on )? cup_on : cup_off ), back, gc, 0, 0,
+	       scale, height, pos, 0 );
   }
   XSetWindowBackgroundPixmap( display, game->cup_box[n], back );
   XFreePixmap( display, back );
   XFreeGC( display, gc );
 }
 
-Game new_game( enum GameMode mode, int *handicap, int preload, int *colors,
-	       int round, int *score, enum Level level ) {
-  int i;
+Game new_game( enum GameMode mode,
+	       RuleSet_t *ruleset,
+	       int *colors,
+	       int round,
+	       int *score,
+	       enum Level level ) {
+
+  int i, gw, gh, bw, bh, bs;
   unsigned long gcm;
   XGCValues gcv;
+  GC gc1;
+  GC gc2;
   Game game = (Game) xmalloc( sizeof( struct _Game ));
   game->state = PLAYING;
   game->mode = mode;
@@ -135,62 +183,98 @@ Game new_game( enum GameMode mode, int *handicap, int preload, int *colors,
     game->player_right[i] = 0;
     game->player_fire[i] = 0;    
     game->computer_state[i] = IDLE;
-    game->computer_thinking_time[i] = 1000;
+    game->computer_thinking_time[i] = COMPUTER_DELAY;
     game->canon_angle[i] = 0.0;
   }
+  game->need_restart_thinking = 0;
   game->multi_player = ( mode != SINGLE_PLAYER );
-  game->nb_boards = ( game->multi_player )? 2 : 1;
+
+  game->nb_boards = ( game->multi_player ? 2 : 1 );
   /* create board(s) */
   for ( i = 0; i < game->nb_boards; i++ ) {
-    game->board[i] = new_board( PERIOD, handicap[i], ! preload );
+
+    game->board[i] = new_board( PERIOD, 
+				&(ruleset[i]),
+				( ! game->multi_player ),
+				colors );
+
     game->opponent[i] = new_opponent( game->board[i], level );
-    if ( preload )
-      load_bubbles( game->board[i], colors );
   }
+  gw = game_win_width(scale);
+  gh = game_win_height(scale);
+  bw = board_win_width(scale);
+  bh = board_win_height(scale);
+  bs = board_spacing(scale);
+
   /* one board in single player mode */
   if ( ! game->multi_player ) {
-    game->board_x[0] = game_win_width(scale)/2 - board_win_width(scale)/2;
-    game->board_y[0] = game_win_height(scale)/2 - board_win_height(scale)/2 +
-      board_spacing(scale)/2;
+    game->board_x[0] = gw/2 - bw/2;
+    game->board_y[0] = gh/2 - bh/2 + bs/2;
   }
   else { /* two boards in multi-player mode */
-    game->board_x[1] = game_win_width(scale)/2 - board_win_width(scale) -
-      board_spacing(scale)/2;
-    game->board_y[1] = game_win_height(scale)/2 - board_win_height(scale)/2 +
-      board_spacing(scale)/2;
-    game->board_x[0] = game_win_width(scale)/2 + board_spacing(scale)/2;
+    game->board_x[1] = gw/2 - bw - bs/2;
+    game->board_y[1] = gh/2 - bh/2 + bs/2;
+    game->board_x[0] = gw/2 + bs/2;
     game->board_y[0] = game->board_y[1];
   }
   gcm = GCFunction | GCGraphicsExposures;
   gcv.graphics_exposures = False;
   gcv.function = GXcopy;
   game->board_gc = XCreateGC( display, root, gcm, &gcv);
+  gc1 = XCreateGC( display, root, gcm, &gcv);
+  gc2 = XCreateGC( display, root, gcm, &gcv);
   /* create windows */
   for ( i = 0; i < game->nb_boards; i++ ) {
-    game->cup_box[i] = 
-      XCreateSimpleWindow( display, win, game->board_x[i],
-			   game->board_y[i] - 5*board_spacing(scale)/3, 
-			   2*scale, 4*board_spacing(scale)/3, 0, 0, 0 );
+    if ( game->mode == DEMO )
+      game->score[i] = 0;
+    game->cup_box[i] = XCreateSimpleWindow( display, 
+					    win,
+					    game->board_x[i],
+					    game->board_y[i] - 5*bs/3, 
+					    2*scale,
+					    4*bs/3,
+					    0, 0, 0 );
     display_cups( game, i, False );
-    game->tags[i] = create_dialog( game->board_x[i] + board_win_width(scale)/2,
-				   game->board_y[i] - board_spacing(scale),
-				   player_name[mode][i], menu_font,
-				   get_pixel( 0xff, 0xff, 0xff ), 0, 1.0 );
-    game->board_window[i] =
-      XCreateSimpleWindow( display, win, game->board_x[i],
-			   game->board_y[i], board_win_width(scale),
-			   board_win_height(scale), 0, 0, 0 );
-    game->board_pixmap[i] = XCreatePixmap( display, root, 
-					   board_win_width(scale),
-					   board_win_height(scale), depth );
+    
+    game->tags[i] = create_dialog( game->board_x[i] + bw/2,
+				   game->board_y[i] - bs,
+				   gettext(player_name[mode][i]),
+				   menu_font,
+				   get_pixel( menufont_color[0], menufont_color[1], 
+				    menufont_color[2] ), 0, 1.0 );
+
+    game->board_window[i] = XCreateSimpleWindow( display, 
+						 win,
+						 game->board_x[i],
+						 game->board_y[i],
+						 bw, bh, 0, 0, 0 );
+
+    game->board_pixmap[i] = XCreatePixmap( display, root, bw, bh, depth );
+
+    /* make board background (position dependant) */
+    game->board_bg[i] = XCreatePixmap( display, root, bw, bh, depth );
+    /* first draw window background */
+    XSetFillStyle( display, gc1, FillTiled );
+    XSetTile( display, gc1, win_bg );
+    XSetTSOrigin( display, gc1, -game->board_x[i], -game->board_y[i] );
+    XFillRectangle( display, game->board_bg[i], gc1, 0, 0, bw, bh );
+    /* then draw decoration */
+    XSetClipMask( display, gc2, board_decoration_mask );
+    XSetClipOrigin( display, gc2, 0, 0 );
+    XCopyArea( display, board_decoration, game->board_bg[i], gc2,
+	       0, 0, bw, bh, 0, 0 );
+
     draw_sprite_pool( get_board_sprite_pool(game->board[i]), 
-		      game->board_pixmap[i], board_bg, board_win_width(scale),
-		      board_win_height(scale));
+		      game->board_pixmap[i],
+		      game->board_bg[i], bw, bh );
+
     XSelectInput( display, game->board_window[i], ExposureMask );
     XSetWindowBackgroundPixmap( display, game->board_window[i], None );
     XMapWindow( display, game->board_window[i] );
     XMapWindow( display, game->cup_box[i] );
   }
+  XFreeGC( display, gc1);
+  XFreeGC( display, gc2);
   return game;
 }
 
@@ -204,6 +288,7 @@ void delete_game( Game game, int no_clean ) {
     delete_opponent( game->opponent[i] );
     delete_board( game->board[i] );
     XFreePixmap( display, game->board_pixmap[i] );
+    XFreePixmap( display, game->board_bg[i] );   
     XDestroyWindow( display, game->board_window[i] );
     XDestroyWindow( display, game->cup_box[i] );
     XDestroyWindow( display, game->tags[i] );
@@ -213,34 +298,20 @@ void delete_game( Game game, int no_clean ) {
     XSetWindowBackgroundPixmap( display, win, win_bg );
 }
 
-static void show_board_msg( Game game, int i, char *msg ) {
-  game->msg_box[i] = create_dialog( game->board_x[i]+board_win_width(scale)/2,
-				    game->board_y[i]+board_win_height(scale)/2,
-				    msg, dialog_font, 
-				    get_pixel( 0xff, 0xff, 0xff ), 2, 1.5 );
-}
-
-static void hide_game_msg( Game game ) {
-  int i;
-  for ( i = 0; i < game->nb_boards; i++ )
-    XDestroyWindow( display, game->msg_box[i] );
-}
-
 static void redraw_game_window( Game game ) {
-  int i, n;
-  Rectangle *r;
-  RectangleList rl;
+  int n;
+  GC gc;
   SpritePool sp;
   for ( n = 0; n < game->nb_boards; n++ ) {
     sp = get_board_sprite_pool( game->board[n] );
-    /* only redraw some rectangles */
-    rl = get_sprite_pool_redraw_list(sp);
-    for ( i = 0; i < rl->size; i++ ) {
-      r = &rl->element[i];
-      XCopyArea( display, game->board_pixmap[n], game->board_window[n], 
-		 game->board_gc, r->x1, r->y1, r->x2-r->x1, r->y2-r->y1,
-		 r->x1, r->y1 );
-    }
+    gc = get_sprite_pool_redraw_gc(sp);
+    XCopyArea( display, game->board_pixmap[n], game->board_window[n], gc,
+	       0,
+	       0,
+	       board_win_width(scale),
+	       board_win_height(scale),
+	       0,
+	       0 );
   }
 }
 
@@ -263,10 +334,16 @@ static void process_x_events( Game game ) {
       /* redraw board windows only, game window has a background pixmap */
       for ( i = 0; i < game->nb_boards; i++ )
 	if ( window == game->board_window[i] ) 
-	  XCopyArea( display, game->board_pixmap[i], game->board_window[i], 
-		     game->board_gc, event.xexpose.x, event.xexpose.y, 
-		     event.xexpose.width, event.xexpose.height, 
-		     event.xexpose.x, event.xexpose.y );
+	  XCopyArea( display,
+		     game->board_pixmap[i],
+		     game->board_window[i], 
+		     game->board_gc,
+		     event.xexpose.x,
+		     event.xexpose.y, 
+		     event.xexpose.width,
+		     event.xexpose.height, 
+		     event.xexpose.x,
+		     event.xexpose.y );
       break;
       
     case KeyRelease:
@@ -303,6 +380,8 @@ static void process_x_events( Game game ) {
       /* escape & pause keys */
       if ( keysym == XK_Escape )
 	game->escape_pressed = 1;
+      if ( keysym == XK_q )
+	game->escape_pressed = 1;
       if ( keysym == XK_p )
 	game->pause_pressed = 1;
       break;
@@ -314,100 +393,131 @@ static void process_x_events( Game game ) {
 }
 
 static void animate_game( Game game, int dt ) {
-  int i, color;
+  int i, color, lost=0;
   for ( i = 0; i < game->nb_boards; i++ ) { 
     animate_board( game->board[i], dt );
+    lost = lost || get_board_state( game->board[i] ) == LOST;
     redraw_sprite_pool( get_board_sprite_pool(game->board[i]), 
-			game->board_pixmap[i], board_bg );
+			game->board_pixmap[i], game->board_bg[i] );
   }
-  if (( game->multi_player )&&( game->state != OVER )) {
-    /* exchange bubbles */
-    while ( receive_bubble(game->board[0]) >= 0 ) {
+  if (( game->multi_player )&&( game->state != OVER )&&( !lost )) {
+    int given = 0;
+    /* exchange bubbles if nobody is dead */
+    while ( (color = give_bubble(game->board[0])) >= 0 ) {
+      given = 1;
       color = rnd( NB_COLORS );
-      send_bubble( game->board[1], color);
+      get_bubble( game->board[1], color);
     }
-    while ( receive_bubble(game->board[1]) >= 0 ) {
+    if (given) {
+      recompute_malus_indicator( game->board[1] );
+      given = 0;
+    }
+     
+    while ( (color = give_bubble(game->board[1])) >= 0 ) {
+      given = 1;
       color = rnd( NB_COLORS );
-      send_bubble( game->board[0], color);
+      get_bubble( game->board[0], color);
+    }
+    if (given) {
+      recompute_malus_indicator( game->board[0] );
     }
   }
 }
 
 static void player_move( Game game, int n ) {
-  if ( game->player_left[n] )
-    rotate_canon_left( game->board[n] );
-  if ( game->player_right[n] )
-    rotate_canon_right( game->board[n] );
-  if ( game->player_fire[n] )
-    fire_board_canon(game->board[n]);
+  if ( game->player_left[n] )  {
+    canon_rotate_left( game->board[n] );
+  } else if ( game->player_right[n] )  {
+    canon_rotate_right( game->board[n] );
+  } else if ( game->player_fire[n] )  {
+    canon_fire(game->board[n]);
+  } else {
+    canon_stop(game->board[n]);
+  }
 }
 
 static void computer_move( Game game, int n ) {
   int shift;
   /* if board not stable then restart from beginning */
-  if (( ! ready_to_fire( game->board[n] ))||
-      ( board_was_lowered( game->board[n] )))
+  if (( get_board_state( game->board[n] ) != READY_TO_FIRE )||
+      ( board_was_lowered( game->board[n] ))) {
+    if ( game->computer_state[n] == THINKING )
+      game->need_restart_thinking = 1;
     game->computer_state[n] = IDLE;
+  }
   game->computer_thinking_time[n] -= frame_duration/1000;
+
   /* think only when board is stable */
-  if ( ready_to_fire( game->board[n] )) {
+  if ( get_board_state( game->board[n] ) == READY_TO_FIRE ) {
     switch ( game->computer_state[n] ) {
+
     case IDLE:
-      reset_opponent( game->opponent[n], game->board[n]);
       game->computer_state[n] = THINKING;
-      break;    
-    case THINKING:
-      if ( find_best_angle(game->opponent[n])) {
-	game->board_best_angle[n] = get_best_angle(game->opponent[n]);
-	game->board_best_eval[n] = get_best_eval(game->opponent[n]);
-	game->computer_state[n] = AIMING;
-      }
       break;
+
+    case THINKING:
+      /* do nothing, computer_think() will switch to AIMING state */
+      break;
+      
     case AIMING:
-      /* if we are losing then wait */
-      if ( game->board_best_eval[n] < 5 )
-	break;
       /* if canon is correctly aimed and thinking time is elapsed then fire */
       if (( get_canon_angle( game->board[n] ) == game->board_best_angle[n] )&&
-	  (( game->mode != PLAYER_VS_COMPUTER )||
-	   ( game->computer_thinking_time[n] <= 0 ))) {
-	fire_board_canon( game->board[n] );
-	/* adjust our pace to our opponent's */
-	game->computer_thinking_time[n] = 
-	  get_last_fire_delay(game->board[0]) + 500;
+	  ( game->computer_thinking_time[n] <= 0 )&&
+	  /* if we are losing then we just wait */
+	  ( game->board_best_eval[n] > 4 )) {
+	canon_fire( game->board[n] );
+	if ( game->mode == PLAYER_VS_COMPUTER )
+	  /* roughly adjust our pace to our opponent's */
+	  game->computer_thinking_time[n] = 
+	    get_last_fire_delay(game->board[0]) + 500; /* 500 = canon load */
+	else
+	  game->computer_thinking_time[n] = COMPUTER_DELAY;
       }
       /* else adjust canon angle */
       shift = game->board_best_angle[n] - get_canon_angle(game->board[n]);
       if ( shift != 0 ) {
 	if ( shift > 0 )
-	  rotate_canon_right( game->board[n] );
+	  canon_rotate_right( game->board[n] );
 	else
-	  rotate_canon_left( game->board[n] );
+	  canon_rotate_left( game->board[n] );
 	/* if necessary, fine tune canon rotation ( yes that's cheating ) */
 	if ( abs(shift) < ( frame_duration/1000.0 )*CANON_ROTATING_SPEED ) {
-	  move_board_canon( game->board[n], 
+	  canon_move( game->board[n], 
 			    (int) floor( abs(shift)/CANON_ROTATING_SPEED ));
 	  canon_stop( game->board[n] );
 	}
+      } else  {
+	canon_stop( game->board[n] );
       }
     }
   }
 }
 
 static void frame_update() {
-  int i, overflow1, overflow2;
+  int i;
+  int lost1;
+  int lost2;
   Game game = current_game;
   if ( game == NULL )
     return;
+
   process_x_events(game);
   switch ( game->state ) {
     
   case PLAYING:
+    redraw_game_window(game);
+
     /* pause */
     if ( game->pause_pressed ) {
       game->state = PAUSED;
       for ( i = 0; i < game->nb_boards; i++ )
-	show_board_msg( game, i, "Pause" );
+	game->msg_box[i] =
+	  create_dialog( game->board_x[i] + board_win_width(scale)/2,
+			 game->board_y[i] + board_win_height(scale)/2,
+			 _("Pause"),
+			 dialog_font, 
+			 get_pixel( menufont_color[0], menufont_color[1], 
+				    menufont_color[2] ), 2, 1.5 );
       break;
     }
     /* abort with ESC key (or any key in demo mode) */
@@ -436,32 +546,39 @@ static void frame_update() {
       break;
     }
     animate_game( game, frame_duration/1000 );
-    redraw_game_window(game);
 
     /* check if game is not over */
     if ( game->multi_player ) {
-      overflow1 = board_overflow( game->board[0] );
-      overflow2 = board_overflow( game->board[1] );
-      if ( overflow1 || overflow2 ) {
+      lost1 = ( get_board_state( game->board[0] ) == LOST );
+      lost2 = ( get_board_state( game->board[1] ) == LOST );
+      if ( lost1 || lost2 ) {
+	canon_stop( game->board[0] );
+	canon_stop( game->board[1] );
 	game->state = OVER;
 	game->result = DRAW;
-	if ( ! overflow1 ) {
+	if ( ! lost1 ) {
 	  game->result = PLAYER1_WON;
 	  explode_board( game->board[0] );
 	}
-	if ( ! overflow2 ) {
+	if ( ! lost2 ) {
 	  game->result = PLAYER1_LOST;
 	  explode_board( game->board[1] );
-	}
+	} 
+	/* stop computer thinking */
+	game->need_restart_thinking = 1;
+	game->computer_state[0] = IDLE;
+	game->computer_state[1] = IDLE;
       }
     }
     else { /* single player mode */
       if ( board_empty( game->board[0] )) {
+	canon_stop( game->board[0] );
 	game->state = OVER;
 	game->result = PLAYER1_WON;
+	freeze_board( game->board[0] );
 	break;
       }
-      if ( board_overflow( game->board[0] )) {
+      if ( get_board_state( game->board[0] ) == LOST ) {
 	game->state = OVER;
 	game->result = PLAYER1_LOST;
       }
@@ -471,157 +588,210 @@ static void frame_update() {
   case PAUSED:
     if ( game->pause_pressed ) {
       game->state = PLAYING;
-      hide_game_msg(game);
+      for ( i = 0; i < game->nb_boards; i++ )
+	XDestroyWindow( display, game->msg_box[i] );
     }
     break;
 
   case OVER:
-    /* wait for end of board animations ( quiet boards ) */
-    if (( board_quiet( game->board[0] )) &&
-	(( ! game->multi_player )||( board_quiet( game->board[1]))))
+    redraw_game_window(game);
+    /* wait for end of board animations */
+    if (( get_board_state( game->board[0] ) == FROZEN )&&
+	(( ! game->multi_player )||
+	 ( get_board_state( game->board[1] ) == FROZEN )))
       game->state = FINISHED;
-    else {
+    else
       animate_game( game, frame_duration/1000 );
-      redraw_game_window(game);
-    }
     break;
     
   default:
     break;
   }
+  /* wait for X server */
+  XSync( display , False );
+  
+  /* to restart computer thinking we do a non-local exit */
+  if ( game->need_restart_thinking ) {
+    game->need_restart_thinking = 0;
+    siglongjmp( restart_thinking, 1 );
+  }
 }
 
-static void timer_sleep( long ms ) {
-  long i;
-  for ( i = ms*fps/1000; i > 0; i-- )
-    pause();
+static void show_board_msg( Game game, int i, char *msg ) {
+  game->msg_box[i] =
+    create_dialog( game->board_x[i] + board_win_width(scale)/2,
+		   game->board_y[i] + board_win_height(scale)/2,
+		   msg,
+		   dialog_font, 
+		   get_pixel( menufont_color[0], menufont_color[1], 
+				    menufont_color[2] ), 2, 1.5 );
 }
 
-static void block_timer() {
-  sigset_t set;
-  sigemptyset( &set );
-  sigaddset( &set, SIGALRM );
-  sigprocmask( SIG_BLOCK, &set, NULL );
-}
-
-static void unblock_timer() {
-  sigset_t set;
-  sigemptyset( &set );
-  sigaddset( &set, SIGALRM );
-  sigprocmask( SIG_UNBLOCK, &set, NULL );
+static void hide_game_msg( Game game ) {
+  int i;
+  for ( i = 0; i < game->nb_boards; i++ )
+    XDestroyWindow( display, game->msg_box[i] );
 }
 
 static void show_start_boxes( Game game ) {
-  static char msg[20];
-  /* enter X critical section */
-  block_timer();
+  static char msg[40];
+
   switch ( game->mode ) {
   case SINGLE_PLAYER:
-    sprintf( msg, "Stage %d", game->round );
+    sprintf( msg, _("Stage %d"), game->round );
     show_board_msg( game, 0, msg );
     break;
   case TWO_PLAYERS:
   case PLAYER_VS_COMPUTER:
-    sprintf( msg, "Round %d", game->round );
+    sprintf( msg, _("Round %d"), game->round );
     show_board_msg( game, 0, msg );
     show_board_msg( game, 1, msg );
     break;
-  default:
-    return;
+  case DEMO:
+    break;
   }
-  unblock_timer();
-  /* sleep for 2 seconds */
-  timer_sleep( 2000 );
-  block_timer();
-  hide_game_msg(game);
-  unblock_timer();
 }
 
 static void show_game_result( Game game ) {
-  int i, on = 1, winner = -1;
-  /* enter X critical section */
-  block_timer();
+  static char msg[40];
+
   switch ( game->result ) {
   case PLAYER1_WON:
-    show_board_msg( game, 0, "Win" );
+    if ( game->mode == SINGLE_PLAYER ) {
+      sprintf( msg, _("Stage %d cleared !"), game->round );
+      show_board_msg( game, 0, msg );
+    } else
+      show_board_msg( game, 0, _("Win") );
     if ( game->multi_player ) {
-      show_board_msg( game, 1, "Lose" );
+      show_board_msg( game, 1, _("Lose") );
       game->score[0]++;
-      winner = 0;
     }
     break;
   case PLAYER1_LOST:
-    show_board_msg( game, 0, "Lose" );
+    show_board_msg( game, 0, _("Lose") );
     if ( game->multi_player ) {
-      show_board_msg( game, 1, "Win" );
+      show_board_msg( game, 1, _("Win") );
       game->score[1]++;
-      winner = 1;
     }
     break;
   case DRAW:
-    show_board_msg( game, 0, "Draw" );
-    show_board_msg( game, 1, "Draw" );
+    show_board_msg( game, 0, _("Draw") );
+    show_board_msg( game, 1, _("Draw") );
     break;
   case ABORTED:
-    return;
+    break;
   }
-  unblock_timer();
-  /* sleep for 4 seconds */
-  for ( i = 0; i < 20; i++ ) {
-    if ( winner >= 0 ) {
+}
+ 
+static void show_blinking_cups( Game game ) {
+  int i;
+  int on = 1;
+  int winner = -1;
+  if ( game->multi_player ) {
+     if ( game->result == PLAYER1_WON )
+       winner = 0;
+     if ( game->result == PLAYER1_LOST )
+       winner = 1;
+  }
+  /* display blinking cup for 3 seconds */
+  for ( i = 0; i < 15; i++ ) {
+    if (( winner >= 0 )&&( game->mode != DEMO )) {
       block_timer();
+       
+      /* Mt: The following does not do what I want (get out of the loop on key press)  
+      process_x_events( game );
+      if (game->escape_pressed || game->key_pressed) {
+	 unblock_timer();
+	 return;
+      }
+       */
+	 
       display_cups( game, winner, on );
-      XClearWindow( display, game->cup_box[winner] );
+       XClearWindow( display, game->cup_box[winner] );
       unblock_timer();
       on = 1 - on;
     }
     timer_sleep(200);
   }
-  block_timer();
-  hide_game_msg(game);
-  unblock_timer();
+}
+
+static void computer_think( Game game ) {
+  int n;
+  int state;
+  int best_eval;
+  int best_angle;
+  for ( n = 0; n < game->nb_boards; n++ ) {
+    block_timer();
+    state = game->computer_state[n];
+    unblock_timer();
+    if ( state == THINKING ) {
+      best_angle = find_best_angle( game->opponent[n], &best_eval );
+      /* switch to AIMING state */
+      block_timer();
+      game->computer_state[n] = AIMING;
+      game->board_best_angle[n] = best_angle;
+      game->board_best_eval[n] = best_eval;
+      unblock_timer();
+    }
+  }
 }
 
 enum GameResult play_game( Game game ) {
-  struct sigaction action;
-  struct itimerval value;
-  struct timeval interval;
-
-  /* setup interval timer */
-  interval.tv_sec = 0;
-  interval.tv_usec = frame_duration;
-  value.it_interval = interval;
-  value.it_value = interval;
-
-  /* prepare for catching SIGALRM signal */
-  action.sa_handler = frame_update;
-  action.sa_flags = 0;
-  sigemptyset(&(action.sa_mask));
-
+  enum GameState state;
   current_game = game;
   game->state = STOPPED;
+  start_timer( frame_duration, frame_update );
+  /*-------------------- enter danger zone ----------------------*/
 
-  if (( sigaction( SIGALRM, &action, NULL ) != 0 )||
-      ( setitimer( ITIMER_REAL, &value, NULL) < 0 )) {
-    perror( "play_game");
-    exit(1);
+  /* From now on, we'll do libc/libX11 calls and non-local exits in 
+     frame_update(), so we shouldn't do anything dangerous (i.e.
+     non-reentrant) here until our timer is stopped.
+  */
+  if ( game->mode != DEMO ) {
+    block_timer();           /* enter critical section */
+    show_start_boxes(game);
+    unblock_timer();         /* exit critical section */
+    
+    timer_sleep( 1500 );
+    
+    block_timer();
+    hide_game_msg(game);
+    unblock_timer();
   }
-  show_start_boxes(game);
   /* start playing */
+  block_timer();
   game->state = PLAYING;
+  unblock_timer();
 
-  /* loop until game is finished or aborted */
-  while (( game->state != FINISHED )&&( game->state != STOPPED ))
-    pause();
-  if ( game->state == FINISHED )
-    show_game_result(game);
+  /* we jump here when we need to restart the thinking process */
+  sigsetjmp(restart_thinking, 1);
   
-  /* stop timer */
-  interval.tv_usec = 0;
-  value.it_interval = interval;
-  value.it_value = interval;
-  setitimer( ITIMER_REAL, &value, NULL);
+  /* loop until game is finished or aborted */
+  do {
+    /* use our spare time to think */
+    computer_think(game);
+    pause();
+    /* get consistent game state */
+    block_timer();
+    state = game->state;
+    unblock_timer();
+  } while (( state != FINISHED )&&( state != STOPPED ));
+  
+  /* show results */
+  if ( state == FINISHED ) {
+    block_timer();
+    show_game_result(game);
+    unblock_timer();
+    
+    show_blinking_cups(game);
+    
+    block_timer();
+    hide_game_msg(game);
+    unblock_timer();
+  }
 
+  stop_timer();
+  /*---------------------- exit danger zone ---------------------*/
   current_game = NULL;
   return game->result;
 }
