@@ -22,6 +22,8 @@
 #include <stdlib.h>
 #include <png.h>
 
+#include "gettext.h"
+
 /*
  * load_png_file() allocates a byte array and fills it with
  * a contiguous sequence of 4-byte pixels in RGBA format:
@@ -34,12 +36,14 @@
  */
 
 unsigned char * load_png_file( const char *file, 
-			       int *width, int *height, int *has_alpha ) {
+			       int *width,
+			       int *height,
+			       int *has_alpha ) {
   FILE *fd;
   unsigned char *data;
   unsigned char header[8];
-  int  bit_depth, color_type;
-  png_uint_32  png_width, png_height, i, rowbytes;
+  int  bit_depth, color_type, i;
+  png_uint_32  png_width, png_height, rowbytes;
   png_structp png_ptr;
   png_infop info_ptr;
   png_bytep *row_pointers;
@@ -53,7 +57,7 @@ unsigned char * load_png_file( const char *file,
   fread( header, 1, 8, fd );
   if ( ! png_check_sig( header, 8 ) ) {
     fclose(fd);
-    fprintf(stderr,"File %s does not have a valid PNG signature.\n", file);
+    fprintf(stderr,_("File %s does not have a valid PNG signature.\n"), file);
     return NULL;
   }
   /* create PNG structs */
@@ -89,7 +93,8 @@ unsigned char * load_png_file( const char *file,
   if (( color_type == PNG_COLOR_TYPE_GRAY )||
       ( color_type == PNG_COLOR_TYPE_GRAY_ALPHA ))
     png_set_gray_to_rgb(png_ptr);
- 
+
+  /* detect alpha layer */
   if (( info_ptr->color_type & PNG_COLOR_MASK_ALPHA )||
       ( png_get_valid( png_ptr, info_ptr, PNG_INFO_tRNS )))
     *has_alpha = 1;
