@@ -1,10 +1,8 @@
 #ifndef _SPRITE_H
 #define _SPRITE_H
-
 #include <X11/Xlib.h>
 
 #include "utils.h"
-#include "rectangle.h"
 
 typedef struct _Sprite * Sprite;
 typedef struct _SpritePool * SpritePool;
@@ -23,13 +21,12 @@ void increase_sprite_clock( Sprite s, int dt );
 
 SpritePool new_sprite_pool( int nb_layers, int nb_sprites_max );
 void delete_sprite_pool( SpritePool sp );
-void empty_sprite_pool( SpritePool sp );
 
 void add_sprite_to_pool( SpritePool sp, Sprite s );
 void remove_sprite_from_pool( SpritePool sp, Sprite s );
 
 void draw_sprite_pool( SpritePool sp, Pixmap dest, Pixmap bg, int w, int h );
 void redraw_sprite_pool( SpritePool sp, Pixmap pixmap, Pixmap background );
-RectangleList get_sprite_pool_redraw_list( SpritePool sp );
+GC get_sprite_pool_redraw_gc( SpritePool sp );
 
 #endif /* _SPRITE_H */
